package cn.feewee.amap3d.modules

import com.amap.api.location.AMapLocationClient
import com.amap.api.maps.AMapException
import com.amap.api.maps.AMapUtils
import com.amap.api.maps.MapsInitializer
import com.amap.api.maps.model.LatLng
import com.amap.api.maps.model.RoutePara
import com.facebook.react.bridge.Promise
import com.facebook.react.bridge.ReactApplicationContext
import com.facebook.react.bridge.ReactContextBaseJavaModule
import com.facebook.react.bridge.ReactMethod

@Suppress("unused")
class SdkModule(val context: ReactApplicationContext) : ReactContextBaseJavaModule() {
  override fun getName(): String {
    return "AMapSdk"
  }

  @ReactMethod
  fun initSDK(apiKey: String?) {
    apiKey?.let {
      MapsInitializer.setApiKey(it)
      MapsInitializer.updatePrivacyAgree(context, true)
      MapsInitializer.updatePrivacyShow(context, true, true)
      AMapLocationClient.updatePrivacyAgree(context, true)
      AMapLocationClient.updatePrivacyShow(context, true, true)
    }
  }

  @ReactMethod
  fun getVersion(promise: Promise) {
    promise.resolve(MapsInitializer.getVersion())
  }

  @ReactMethod
  fun openAmapNavi(sLat: Double, sLng: Double, eLat: Double, eLng: Double, sName: String?, eName: String?, promise: Promise) {
    try {
      val routePara = RoutePara()
      routePara.startPoint = LatLng(sLat,sLng)
      routePara.endPoint = LatLng(eLat,eLng)
      routePara.startName = sName
      routePara.endName = eName
      AMapUtils.openAMapDrivingRoute(routePara, context)
      promise.resolve(null)
    } catch (ae: AMapException) {
      promise.reject("-2", ae.errorMessage, ae);
    } catch (e:Exception) {
      promise.reject("-1", "未知错误，请重试", e);
    }
  }
}
